//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         Sh. Nourbakhsh Rad
// Remarks:        
// known Problems: none
// Version:        1.0.0   20.01.2010
// Description:    4x4 Matrix Keypad
//
//-----------------------------------------------------------------------------

#include "4Key.h"


//******************* Constants
volatile unsigned char				 		KEY4val = 0x00;


//*************************************************
//******************* Functions *******************
//*************************************************
//--------------------- Defines all ROW pins as input
void KEY4_ROW_in(void)
{
	cbi(KEY4_R1_DDR, KEY4_R1_BIT);
	cbi(KEY4_R2_DDR, KEY4_R2_BIT);
	
	#ifdef KEY4_R3_BIT
		cbi(KEY4_R3_DDR, KEY4_R3_BIT);
	#endif
	#ifdef KEY4_R4_BIT
		cbi(KEY4_R4_DDR, KEY4_R4_BIT);
	#endif
	#ifdef KEY4_R5_BIT
		cbi(KEY4_R5_DDR, KEY4_R5_BIT);
	#endif
}	//KEY4_ROW_in

//--------------------- Read from ROWs
unsigned char KEY4_ROW_read(void)
{
	unsigned char				temp = 0x00;
	
	temp |= (KEY4_R1_state? 0x01 : 0x00);
	temp |= (KEY4_R2_state? 0x02 : 0x00);
	
	#ifdef KEY4_R3_BIT
		temp |= (KEY4_R3_state? 0x04 : 0x00);
	#endif
	#ifdef KEY4_R4_BIT
		temp |= (KEY4_R4_state? 0x08 : 0x00);
	#endif
	#ifdef KEY4_R5_BIT
		temp |= (KEY4_R5_state? 0x10 : 0x00);
	#endif
	
	return temp;
}	//KEY4_ROW_read

//--------------------- Enable ROW pins pullup (Read mode)
void KEY4_ROW_pullup(void)
{
	sbi(KEY4_R1_PRT, KEY4_R1_BIT);
	sbi(KEY4_R2_PRT, KEY4_R2_BIT);

	#ifdef KEY4_R3_BIT
		sbi(KEY4_R3_PRT, KEY4_R3_BIT);
	#endif
	#ifdef KEY4_R4_BIT
		sbi(KEY4_R4_PRT, KEY4_R4_BIT);
	#endif
	#ifdef KEY4_R5_BIT
		sbi(KEY4_R5_PRT, KEY4_R5_BIT);
	#endif
}	//KEY4_ROW_pullup

//--------------------- Disable ROW pins pullup (High-Impedance, Idel mode)
void KEY4_ROW_highz(void)
{
	cbi(KEY4_R1_PRT, KEY4_R1_BIT);
	cbi(KEY4_R2_PRT, KEY4_R2_BIT);

	#ifdef KEY4_R3_BIT
		cbi(KEY4_R3_PRT, KEY4_R3_BIT);
	#endif
	#ifdef KEY4_R4_BIT
		cbi(KEY4_R4_PRT, KEY4_R4_BIT);
	#endif
	#ifdef KEY4_R5_BIT
		cbi(KEY4_R5_PRT, KEY4_R5_BIT);
	#endif
}	//KEY4_ROW_highz


//---------------------------------------------------------------
//--------------------- Defines all COL pins as output
void KEY4_COL_out(void)
{
	sbi(KEY4_C1_DDR, KEY4_C1_BIT);
	sbi(KEY4_C2_DDR, KEY4_C2_BIT);

	#ifdef KEY4_C3_BIT
		sbi(KEY4_C3_DDR, KEY4_C3_BIT);
	#endif
	#ifdef KEY4_C4_BIT
		sbi(KEY4_C4_DDR, KEY4_C4_BIT);
	#endif
	#ifdef KEY4_C5_BIT
		sbi(KEY4_C5_DDR, KEY4_C5_BIT);
	#endif
	#ifdef KEY4_C6_BIT
		sbi(KEY4_C6_DDR, KEY4_C6_BIT);
	#endif
	#ifdef KEY4_C7_BIT
		sbi(KEY4_C7_DDR, KEY4_C7_BIT);
	#endif
	#ifdef KEY4_C8_BIT
		sbi(KEY4_C8_DDR, KEY4_C8_BIT);
	#endif
}	//KEY4_COL_out

//--------------------- Write to COLs
void KEY4_COL_write(unsigned char col)
{
	KEY4_C1((col & 0x01));
	KEY4_C2((col & 0x02));
	
	#ifdef KEY4_C3_BIT
		KEY4_C3((col & 0x04));
	#endif
	#ifdef KEY4_C4_BIT
		KEY4_C4((col & 0x08));
	#endif
	#ifdef KEY4_C5_BIT
		KEY4_C5((col & 0x10));
	#endif
	#ifdef KEY4_C6_BIT
		KEY4_C6((col & 0x20));
	#endif
	#ifdef KEY4_C7_BIT
		KEY4_C7((col & 0x40));
	#endif
	#ifdef KEY4_C8_BIT
		KEY4_C8((col & 0x80));
	#endif		
}	//KEY4_COL_write

//--------------------- High all COL pins
void KEY4_COL_high(void)
{
	sbi(KEY4_C1_PRT, KEY4_C1_BIT);
	sbi(KEY4_C2_PRT, KEY4_C2_BIT);
	
	#ifdef KEY4_C3_BIT
		sbi(KEY4_C3_PRT, KEY4_C3_BIT);
	#endif
	#ifdef KEY4_C4_BIT
		sbi(KEY4_C4_PRT, KEY4_C4_BIT);
	#endif		
	#ifdef KEY4_C5_BIT
		sbi(KEY4_C5_PRT, KEY4_C5_BIT);
	#endif		
	#ifdef KEY4_C6_BIT
		sbi(KEY4_C6_PRT, KEY4_C6_BIT);
	#endif		
	#ifdef KEY4_C7_BIT
		sbi(KEY4_C7_PRT, KEY4_C7_BIT);
	#endif		
	#ifdef KEY4_C8_BIT
		sbi(KEY4_C8_PRT, KEY4_C8_BIT);
	#endif		
}	//KEY4_COL_high

//--------------------- Low all COL pins
void KEY4_COL_low(void)
{
	cbi(KEY4_C1_PRT, KEY4_C1_BIT);
	cbi(KEY4_C2_PRT, KEY4_C2_BIT);

	#ifdef KEY4_C3_BIT
		cbi(KEY4_C3_PRT, KEY4_C3_BIT);
	#endif
	#ifdef KEY4_C4_BIT
		cbi(KEY4_C4_PRT, KEY4_C4_BIT);
	#endif
	#ifdef KEY4_C5_BIT
		cbi(KEY4_C5_PRT, KEY4_C5_BIT);
	#endif
	#ifdef KEY4_C6_BIT
		cbi(KEY4_C6_PRT, KEY4_C6_BIT);
	#endif
	#ifdef KEY4_C7_BIT
		cbi(KEY4_C7_PRT, KEY4_C7_BIT);
	#endif
	#ifdef KEY4_C8_BIT
		cbi(KEY4_C8_PRT, KEY4_C8_BIT);
	#endif
}	//KEY4_COL_low


//---------------------------------------------------------------
//---------------------------------------------------------------
void KEY4_init(void)
{
	KEY4_COL_out();
	KEY4_COL_low();
	KEY4_ROW_in();
	KEY4_ROW_pullup();
	KEY4_INT_init();
}	//KEY4_init

//---------------------------------------------------------------
//---------------------------------------------------------------
ISR_KEY4_INT()
{
	unsigned char 				i;

 	KEY4_INT_dis();

	//---------------------------
	for(i=0; i<COLS_NUMBER; i++)
	{
		KEY4_COL_write(COL_MASK[i]);
	
		if(KEY4_ROW_read() != ROW_MASK[ROWS_NUMBER])
		{	
			KEY4val = KEY4_ROW_read() | (i<<5);
	
			KEY4_COL_low();
			KEY4_INT_ena();
	
			//continue;
			return; 
		}
	}

	//---------------------------
	KEY4val = 0x00;

	KEY4_COL_low();
	KEY4_INT_ena();
}	//ISR_RENC_INT
