//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         Sh. Nourbakhsh Rad
// Remarks:        
// known Problems: none
// Version:        1.0.0   20.01.2010
// Description:    4x4 Matrix Keypad
//
//-----------------------------------------------------------------------------

#ifndef _4x4KEY_H_
	#define _4x4KEY_H_

	#include "4Keyconfig.h"

	
	extern volatile unsigned char				 		KEY4val;						//4x4 Matrix Keypad's global constant

	//---------------------
	static const unsigned char	 						COL_MASK[8] = {
																									0b11111110, 				//1
																									0b11111101, 				//2
																									0b11111011, 				//3
																									0b11110111, 				//4
																									0b11101111, 				//5
																									0b11011111, 				//6
																									0b10111111, 				//7
																									0b01111111	 				//8
																									};
	
	static const unsigned char	 						ROW_MASK[6] = {
																									0b00000000, 				//0
																									0b00000001, 				//1
																									0b00000011, 				//2
																									0b00000111, 				//3
																									0b00001111, 				//4
																									0b00011111, 				//5
																									};


	//---------------------------------------------------------------
	//---------------------------------------------------------------
	#ifndef _BV
		#define _BV(pin)											(1<<pin)
	#endif

	#ifndef sbi
		#define sbi(port, pin)								port |=  _BV(pin)
		#define cbi(port, pin)								port &= ~_BV(pin)
	#endif

	#ifndef inb
		#define inb(port)											port
		#define outb(port, data)							port = data
	#endif
	
	#ifndef bit_is_set
		#define bit_is_set(port,pin)	 				(port & _BV(pin))
	#endif


	//---------------------------------------------------------------
	#define IS_Key_Release									(bit_is_set(KEY4_R1_PIN, KEY4_R1_BIT))
	#define IS_Key_Press										(bit_is_clear(KEY4_R1_PIN, KEY4_R1_BIT))

	//--------------------- Read from each ROWs
	#define KEY4_R1_state										(bit_is_set(KEY4_R1_PIN, KEY4_R1_BIT))
	#define KEY4_R2_state										(bit_is_set(KEY4_R2_PIN, KEY4_R2_BIT))
	
	#ifdef KEY4_R3_BIT
		#define KEY4_R3_state									(bit_is_set(KEY4_R3_PIN, KEY4_R3_BIT))
	#endif
	#ifdef KEY4_R4_BIT
		#define KEY4_R4_state									(bit_is_set(KEY4_R4_PIN, KEY4_R4_BIT))
	#endif
	#ifdef KEY4_R5_BIT
		#define KEY4_R5_state									(bit_is_set(KEY4_R5_PIN, KEY4_R5_BIT))
	#endif


	//---------------------------------------------------------------
	//--------------------- Write to each COLs
	#define KEY4_C1(x)											(x ? (sbi(KEY4_C1_PRT, KEY4_C1_BIT)) : (cbi(KEY4_C1_PRT, KEY4_C1_BIT)))
	#define KEY4_C2(x)											(x ? (sbi(KEY4_C2_PRT, KEY4_C2_BIT)) : (cbi(KEY4_C2_PRT, KEY4_C2_BIT)))

	#ifdef KEY4_C3_BIT
		#define KEY4_C3(x)										(x ? (sbi(KEY4_C3_PRT, KEY4_C3_BIT)) : (cbi(KEY4_C3_PRT, KEY4_C3_BIT)))
	#endif
	#ifdef KEY4_C4_BIT
		#define KEY4_C4(x)										(x ? (sbi(KEY4_C4_PRT, KEY4_C4_BIT)) : (cbi(KEY4_C4_PRT, KEY4_C4_BIT)))
	#endif
	#ifdef KEY4_C5_BIT
		#define KEY4_C5(x)										(x ? (sbi(KEY4_C5_PRT, KEY4_C5_BIT)) : (cbi(KEY4_C5_PRT, KEY4_C5_BIT)))
	#endif
	#ifdef KEY4_C6_BIT
		#define KEY4_C6(x)										(x ? (sbi(KEY4_C6_PRT, KEY4_C6_BIT)) : (cbi(KEY4_C6_PRT, KEY4_C6_BIT)))
	#endif
	#ifdef KEY4_C7_BIT
		#define KEY4_C7(x)										(x ? (sbi(KEY4_C7_PRT, KEY4_C7_BIT)) : (cbi(KEY4_C7_PRT, KEY4_C7_BIT)))
	#endif
	#ifdef KEY4_C8_BIT
		#define KEY4_C8(x)										(x ? (sbi(KEY4_C8_PRT, KEY4_C8_BIT)) : (cbi(KEY4_C8_PRT, KEY4_C8_BIT)))
	#endif


	//---------------------------------------------------------------
	#if USE_4K_INT
		#define KEY4_INT_PRT									KEY4_R1_PRT
		#define KEY4_INT_DDR									KEY4_R1_DDR
		#define KEY4_INT_PIN									KEY4_R1_PIN
		#define KEY4_INT_BIT									KEY4_R1_BIT
	
		#define KEY4_IPIN_init()							cbi(KEY4_INT_DDR, KEY4_INT_BIT); 	sbi(KEY4_INT_PRT, KEY4_INT_BIT)

		//Initialize 4x4 Matrix Keypad interrupt
		#define KEY4_INT_init()								KEY4_IPIN_init();	\
																					KEY4_INT_Sctrl();	\
																					KEY4_INT_ena()
		
	//---------------------	
	#else
		//#define ISR_KEY4_INT()								void ISR_KEY4_INT(void)

		#define KEY4_INT_init()
	#endif


	//******************* Function Prototypes
	void KEY4_init(void);
	ISR_KEY4_INT();
	
#endif //_4x4KEY_H_
