//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         Sh. Nourbakhsh Rad
// Remarks:        
// known Problems: none
// Version:        1.0.0   20.01.2010
// Description:    4x4 Matrix Keypad
//
//-----------------------------------------------------------------------------

#ifndef _4x4KEYC_H_
	#define _4x4KEYC_H_

  #include "..\app_config.h"

	
	//----------- Definitions
	#define ROWS_NUMBER											3										//Min: 2 ,  Max: 5
	#define COLS_NUMBER											3										//Min: 2 ,  Max: 8

	#define USE_4K_INT											0										//1: enable INT mode,		0: disable INT mode (pulling mode!)

	// Pins.....
	//----------- 4x4 Keypad ROWs (input)
	#define KEY4_R1_PRT											PORTC								//If USE_4K_INT = 1 then: 4x4 Keypad INTx (input, INT0)
	#define KEY4_R1_DDR											DDRC
	#define KEY4_R1_PIN											PINC
	#define KEY4_R1_BIT											5

	#define KEY4_R2_PRT											PORTC
	#define KEY4_R2_DDR											DDRC
	#define KEY4_R2_PIN											PINC
	#define KEY4_R2_BIT											6

	#define KEY4_R3_PRT											PORTC
	#define KEY4_R3_DDR											DDRC
	#define KEY4_R3_PIN											PINC
	#define KEY4_R3_BIT											7

	#define KEY4_R4_PRT											PORTB
	#define KEY4_R4_DDR											DDRB
	#define KEY4_R4_PIN											PINB
	//#define KEY4_R4_BIT											3

	#define KEY4_R5_PRT											PORTB
	#define KEY4_R5_DDR											DDRB
	#define KEY4_R5_PIN											PINB
	//#define KEY4_R5_BIT											3

	//----------- 4x4 Keypad COLs (output)
	#define KEY4_C1_PRT											PORTC
	#define KEY4_C1_DDR											DDRC
	#define KEY4_C1_PIN											PINC
	#define KEY4_C1_BIT											2

	#define KEY4_C2_PRT											PORTC
	#define KEY4_C2_DDR											DDRC
	#define KEY4_C2_PIN											PINC
	#define KEY4_C2_BIT											3

	#define KEY4_C3_PRT											PORTC
	#define KEY4_C3_DDR											DDRC
	#define KEY4_C3_PIN											PINC
	#define KEY4_C3_BIT											4

	#define KEY4_C4_PRT											PORTB
	#define KEY4_C4_DDR											DDRB
	#define KEY4_C4_PIN											PINB
	//#define KEY4_C4_BIT											7

	#define KEY4_C5_PRT											PORTB
	#define KEY4_C5_DDR											DDRB
	#define KEY4_C5_PIN											PINB
	//#define KEY4_C5_BIT											4

	#define KEY4_C6_PRT											PORTB
	#define KEY4_C6_DDR											DDRB
	#define KEY4_C6_PIN											PINB
	//#define KEY4_C6_BIT											4

	#define KEY4_C7_PRT											PORTB
	#define KEY4_C7_DDR											DDRB
	#define KEY4_C7_PIN											PINB
	//#define KEY4_C7_BIT											4

	#define KEY4_C8_PRT											PORTB
	#define KEY4_C8_DDR											DDRB
	#define KEY4_C8_PIN											PINB
	//#define KEY4_C8_BIT											4

	//---------------------------------------------------------------
	#if USE_4K_INT
		// Define interrupt service functions
		#define	ISR_KEY4_INT()								ISR(INT2_vect)			//4x4 Matrix Keypad interrupt ISR
	
		//---------------------	
		//4x4 Matrix Keypad interrupt sense control (The falling edge of INTx generates an interrupt request)
		#define KEY4_INT_Sctrl()							MCUCSR &= ~_BV(ISC2)
	
		#define KEY4_INT_ena()								GICR   |=  _BV(INT2)
		#define KEY4_INT_dis()								GICR   &= ~_BV(INT2)
	#else
		
		#define KEY4_INT_ena()
		#define KEY4_INT_dis()
	#endif


	//---------------------------------------------------------------
	//			LM											RM
	//
	//														RE
	//				 OK								CL
	//									UP
	//						LF		DN	  RT
	
	#define NoKey														0x00

	//----  R1
	#define KeyLM														0x06			//C1
	#define KeyRE														0x26			//C2
	#define KeyRM														0x46			//C3
	
	//----  R2
	#define KeyOK														0x04			//C1
	#define KeyCL														0x24			//C2
	#define KeyLF														0x44			//C3

	//----  R3
	#define KeyRT														0x02			//C1
	#define KeyDN														0x22			//C2
	#define KeyUP														0x42			//C3

#endif //_4x4KEYC_H_
