//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         Sh. Nourbakhsh Rad based on Fabian Maximilian Thiele KS0108 driver!
// Remarks:        
// known Problems: none
// Version:        1.8.0
// Description:    Graphic Library for KS0108- (and compatible) based LCDs
//								 Tuned for memory mapped GLCD and Persian/Arabic/English font use 06-04-2011
//-----------------------------------------------------------------------------

#ifndef	KS0108C_H
	#define KS0108C_H
	
  #include "..\app_config.h"


	//----------- Definitions
	#define KS108_ORN_PORTRAIT									0						// 0: Landscape display
																													// 1: Portrait display
	
	#define KS108_PORT_INTERFACE								1						// 0: GLCD memory-mapped interfaced
																													// 1: GLCD port interfaced
	
	#define KS108_BKL_MODE											2						// 0: Disable backlight control
																													// 1: ON/OFF backlight control
																													// 2: PWM backlight control
	
	// Pins.....
	#if KS108_PORT_INTERFACE
		//--- DATA ---
		#define KS108_DB_DDR											DDRB				// KS0108 DATA port
		#define KS108_DB_PRT											PORTB
		#define KS108_DB_PIN											PINB

		//--- CTRL ---
		#define KS108_RS_DDR											DDRA				// KS0108  RS (D/I)
		#define KS108_RS_PRT											PORTA
		#define KS108_RS_BIT											2

		#define KS108_RW_DDR											DDRA				// KS0108  R/W
		#define KS108_RW_PRT											PORTA
		#define KS108_RW_BIT											3

		#define KS108_EN_DDR											DDRA				// KS0108  EN
		#define KS108_EN_PRT											PORTA
		#define KS108_EN_BIT											4

		#define KS108_CS1_DDR											DDRA				// KS0108  CSEL1
		#define KS108_CS1_PRT											PORTA
		#define KS108_CS1_BIT											0

		#define KS108_CS2_DDR											DDRA				// KS0108  CSEL2
		#define KS108_CS2_PRT											PORTA
		#define KS108_CS2_BIT											1


	#elif	defined(XMemory_OK)																							//for XMemory_OK refer to app_config.h
	
    #define RAM_ena()													sbi(MCUCR, SRE)						// enable RAM interface (memory mapped glcd)
		#define RAM_WS_1()												sbi(XMCRA, SRW11)					// and one wait state

		// absolute address of LCD Controller #0 CTRL and DATA registers (Write)
		#define W_CONTROL0_CTRL_ADDR							0x8000
		#define W_CONTROL0_DATA_ADDR							0x8200
						//	 AAAA CCRR
						//	 DDDD SSSW
						//   1000 0000 | 0000 0000  ===  	0x8000
						//   1000 0010 | 0000 0000  ===  	0x8200
		// absolute address of LCD Controller #0 CTRL and DATA registers (Read)
		#define R_CONTROL0_CTRL_ADDR							0x8100
		#define R_CONTROL0_DATA_ADDR							0x8300
						//	 AAAA CCRR
						//	 DDDD SSSW
						//   1000 0001 | 0000 0000  ===  	0x8100
						//   1000 0011 | 0000 0000  ===  	0x8300
	
		// offset of other controllers with respect to controller0
		#define CONTROL_ADDR_OFFSET								0x0400
						//	 AAAA CCRR
						//	 DDDD SSSW
						//   0000 0100 | 0000 0000  ===  	0x0400
	#else
		#error "external data memory interface not available for this device, use GLCD port interface mode"
	#endif
		

	//---------------------------------------------------------------
	#define KS108_RST_DDR												DDRA				// KS0108  Reset
	#define KS108_RST_PRT												PORTA
	#define KS108_RST_BIT												5
	
	//----	ON/OFF or PWM backlight control definitions
	#if (KS108_BKL_MODE >0)
		#define KS108_BKL_DDR											DDRD				// KS0108  Backlight
		#define KS108_BKL_PRT											PORTD
		#define KS108_BKL_BIT											4

		//----	PWM backlight control definitions
		#if (KS108_BKL_MODE ==2)
			//Initialize Timer to generate PWM for backlight (Timer1 : Fast PWM, 8bit - mode5, clk/64)
			#define	BKL_PWM_init()									TCCR1A  = 0b00100101;		\
																							TCCR1B  = 0b00001011
			
			#define	BKL_PWM_START()								 	TCNT1   = 0;						\
																							OCR1B   = 0;						\
																							TCCR1B |= 0b00000011
		
			#define	BKL_PWM_STOP()									TCCR1B &= 0b11111000
		
			#define	BKL_PWM_SET(prc)								OCR1B   = prc
		#endif
	#endif


	//---------------------------------------------------------------
	//---------------------------------------------------------------
	#if KS108_PORT_INTERFACE			
		#define KS108_DB_WR(x)										(KS108_DB_PRT = x)
		#define KS108_DB_RD												 KS108_DB_PIN

		#define KS108_DB_OUT()										(KS108_DB_DDR = 0xFF)
		#define KS108_DB_IN()											(KS108_DB_DDR = 0x00)

		#define KS108_DB_CLR()										(KS108_DB_PRT = 0x00)
		#define KS108_DB_PLU()										(KS108_DB_PRT = 0xFF)

		#define KS108_DB_init()										KS108_DB_OUT(); 	KS108_DB_CLR()

		//-------------------------
		#define KS108_CS1_init()									sbi(KS108_CS1_DDR, KS108_CS1_BIT); cbi(KS108_CS1_PRT, KS108_CS1_BIT)
		#define KS108_CS2_init()									sbi(KS108_CS2_DDR, KS108_CS2_BIT); cbi(KS108_CS2_PRT, KS108_CS2_BIT)
		#define KS108_RS_init()										sbi(KS108_RS_DDR,  KS108_RS_BIT);  cbi(KS108_RS_PRT,  KS108_RS_BIT)
		#define KS108_RW_init()										sbi(KS108_RW_DDR,  KS108_RW_BIT);  cbi(KS108_RW_PRT,  KS108_RW_BIT)
		#define KS108_EN_init()										sbi(KS108_EN_DDR,  KS108_EN_BIT);  cbi(KS108_EN_PRT,  KS108_EN_BIT)

		#define KS108_CS1(x)											(x ? (sbi(KS108_CS1_PRT, KS108_CS1_BIT)) : (cbi(KS108_CS1_PRT, KS108_CS1_BIT)))
		#define KS108_CS2(x)											(x ? (sbi(KS108_CS2_PRT, KS108_CS2_BIT)) : (cbi(KS108_CS2_PRT, KS108_CS2_BIT)))
		#define KS108_RS(x)												(x ? (sbi(KS108_RS_PRT,  KS108_RS_BIT))  : (cbi(KS108_RS_PRT,  KS108_RS_BIT)))
		#define KS108_RW(x)												(x ? (sbi(KS108_RW_PRT,  KS108_RW_BIT))  : (cbi(KS108_RW_PRT,  KS108_RW_BIT)))
		#define KS108_EN(x)												(x ? (sbi(KS108_EN_PRT,  KS108_EN_BIT))  : (cbi(KS108_EN_PRT,  KS108_EN_BIT)))
	
		#define KS108_CS(cp)											{if			(cp==0)	{KS108_CS1(HIGH);	KS108_CS2(LOW);}	\
																							 else if(cp==1)	{KS108_CS1(LOW); 	KS108_CS2(HIGH);}}
	#endif
	
	//-------------------------
	#ifdef KS108_RST_BIT
		#define KS108_RST_init()									sbi(KS108_RST_DDR, KS108_RST_BIT); cbi(KS108_RST_PRT, KS108_RST_BIT)
		#define KS108_RST(x)											(x ? (sbi(KS108_RST_PRT, KS108_RST_BIT)) : (cbi(KS108_RST_PRT, KS108_RST_BIT)))
	
	#else
		#define KS108_RST_init()
		#define KS108_RST(x)
	
	#endif
	
	//----	Disable backlight control
	#if (KS108_BKL_MODE ==0)
		#define KS108_BKL_init()
		#define KS108_BKL(x)
	
	//----	ON/OFF backlight control
	#elif (KS108_BKL_MODE ==1)
		#define KS108_BKL_init()									sbi(KS108_BKL_DDR, KS108_BKL_BIT); cbi(KS108_BKL_PRT, KS108_BKL_BIT)
		#define KS108_BKL(x)                      (x ? (sbi(KS108_BKL_PRT, KS108_BKL_BIT)) : (cbi(KS108_BKL_PRT, KS108_BKL_BIT)))
	
	//----	PWM backlight control
	#elif (KS108_BKL_MODE ==2)
		#define KS108_BKL_init()									BKL_PWM_init();											\
																							sbi(KS108_BKL_DDR, KS108_BKL_BIT); 	\
																							cbi(KS108_BKL_PRT, KS108_BKL_BIT);	\
																							BKL_PWM_START()
																							
		#define KS108_BKL(n)											BKL_PWM_SET(n)
	
	#endif
	
#endif	//KS0108C_H
