//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         Sh. Nourbakhsh Rad
// Remarks:        
// known Problems: none
// Version:        1.8.0
// Description:    GLCD's graphic founctions
//-----------------------------------------------------------------------------

#include "graphic.h"


// Description	: Draw a line on a graphic LCD
// Input        : x1,y1 - starting coordinates
//              : x2,y2 - ending coordinates
//********************************************************************
void Line(Guint x1, Guint y1, Guint x2, Guint y2, Guint color)
{
	signed int  	x, y, addx, addy, dx, dy;
	signed long 	P;
	int 					i;

	dx = abs((signed int)(x2 - x1));
	dy = abs((signed int)(y2 - y1));
	x  = x1;
	y  = y1;

	addx = addy = 1;
	if(x1 > x2)		addx = -1;
	if(y1 > y2)		addy = -1;

	if (dx >= dy)
	{
		P = 2L*dy - dx;
		for (i=0; i<=dx; ++i)
		{
	  	PutPixel((Guint)x, (Guint)y, color);
			if(P < 0)
	  	{
	    	P += 2*dy;
	    	x += addx;
	  	}
			else
			{
	    	P += 2*dy - 2*dx;
	    	x += addx;
	    	y += addy;
	  	}
		}
	}
	else
	{
		P = 2L*dx - dy;
		for(i=0; i<=dy; ++i)
		{
	  	PutPixel((Guint)x, (Guint)y, color);
	  	if(P < 0)
	  	{
	    	P += 2*dx;
	    	y += addy;
	  	}
	  	else
	  	{
	    	P += 2*dx - 2*dy;
	    	x += addx;
	    	y += addy;
	  	}
		}
	}
}	//*Line

// Description  : Draw a rectangle on a graphic LCD
// Input        : x1,y1 - starting coordinates
//              : x2,y2 - ending coordinates
//********************************************************************
void Rectangle(Guint x1, Guint y1, Guint x2, Guint y2, Guint color)
{
		RectangleFill(x1, y1, x2, y1, color);    // top
    RectangleFill(x1, y2, x2, y2, color);    // bottom
    RectangleFill(x1, y1, x1, y2, color);    // left
    RectangleFill(x2, y1, x2, y2, color);    // right
}	//*Rectangle

// Description	: Draws a beveled figure on the screen. 
// Input       	: x1, y1 - coordinate position of the upper left center
//		      		: x2, y2 - coordinate position of the lower right center
//             	: rad    - defines the redius of the circle,
//             	: fill   - fill yes or no
//********************************************************************
void RoundRectangle(Guint x1, Guint y1, Guint x2, Guint y2, Guint radius, unsigned char fill, Guint color)
{
	signed int 		a, b, P;

	a  = 0;       				// increment by 1
	b  = radius;  				// decrement by 1 using P
	P  = 1 - (signed int)(radius);

	if (fill)
	{		
		RectangleFill((Guint)(x1), (Guint)(y1+radius), (Guint)(x2), (Guint)(y2-radius), color);
		
		do
		{
      RectangleFill((Guint)(x1-a+radius), (Guint)(y1-b+radius), (Guint)(a+x2-radius), (Guint)(y1-b+radius), color);	// 8 --> 1
      RectangleFill((Guint)(x1-b+radius), (Guint)(y1-a+radius), (Guint)(b+x2-radius), (Guint)(y1-a+radius), color);	// 7 --> 2
      RectangleFill((Guint)(x1-b+radius), (Guint)(a+y2-radius), (Guint)(b+x2-radius), (Guint)(a+y2-radius), color);	// 6 --> 3
			RectangleFill((Guint)(x1-a+radius), (Guint)(b+y2-radius), (Guint)(a+x2-radius), (Guint)(b+y2-radius), color);	// 5 --> 4

			if(P < 0)		P += 3 +2 *a++;
			else				P += 5 +2 *(a++ - b--);
		} while(a <= b);
	} //fill
	else
	{
		RectangleFill((Guint)(x1+radius), (Guint)(y1)       , (Guint)(x2-radius), (Guint)(y1)       , color);	// top
		RectangleFill((Guint)(x1+radius), (Guint)(y2)       , (Guint)(x2-radius), (Guint)(y2)       , color);	// bottom
		RectangleFill((Guint)(x1)       , (Guint)(y1+radius), (Guint)(x1)       , (Guint)(y2-radius), color);	// left
		RectangleFill((Guint)(x2)       , (Guint)(y1+radius), (Guint)(x2)       , (Guint)(y2-radius), color);	// right

		do
		{
			PutPixel((Guint)(a+x2-radius), (Guint)(y1-b+radius), color);	// `````` Segment 1
			PutPixel((Guint)(b+x2-radius), (Guint)(y1-a+radius), color);	// `````` Segment 2
			
			PutPixel((Guint)(b+x2-radius), (Guint)(a+y2-radius), color);	// `````` Segment 3
			PutPixel((Guint)(a+x2-radius), (Guint)(b+y2-radius), color);	// `````` Segment 4
			
			PutPixel((Guint)(x1-a+radius), (Guint)(b+y2-radius), color);	// `````` Segment 5
			PutPixel((Guint)(x1-b+radius), (Guint)(a+y2-radius), color);	// `````` Segment 6
			
			PutPixel((Guint)(x1-b+radius), (Guint)(y1-a+radius), color);	// `````` Segment 7
			PutPixel((Guint)(x1-a+radius), (Guint)(y1-b+radius), color);	// `````` Segment 8
			
			if(P < 0)		P += 3 +2 *a++;
			else				P += 5 +2 *(a++ - b--);
		} while(a <= b);
	} //no fill
}	//RoundRectangle

// Description	: Draw a circle on the screen. 
// Input       	: x,y  - the center of the circle
//             	: rad  - defines the redius of the circle,
//             	: fill - fill yes or no
//********************************************************************
void Circle_Fill(Guint x, Guint y, Guint radius, unsigned char fill, Guint color)
{
	signed int 	a, b, P;
   
	a = 0;
	b = radius;
	P = 1 - (signed int)(radius);

	do
	{
		if(fill)
		{
			RectangleFill((Guint)(x-a), (Guint)(y+b), (Guint)(x+a), (Guint)(y+b), color);	// 5 --> 4
			RectangleFill((Guint)(x-a), (Guint)(y-b), (Guint)(x+a), (Guint)(y-b), color);	// 8 --> 1
			RectangleFill((Guint)(x-b), (Guint)(y+a), (Guint)(x+b), (Guint)(y+a), color);	// 6 --> 3
			RectangleFill((Guint)(x-b), (Guint)(y-a), (Guint)(x+b), (Guint)(y-a), color);	// 7 --> 2
		}
		else
		{
																										//       		8	1
																										//       	7			2
																										//       	6			3
																										//       		5	4
																										//
			PutPixel((Guint)(a+x), (Guint)(b+y), color);	// `````` Segment 4
			PutPixel((Guint)(b+x), (Guint)(a+y), color);	// `````` Segment 3
			PutPixel((Guint)(x-a), (Guint)(b+y), color);	// `````` Segment 5
			PutPixel((Guint)(x-b), (Guint)(a+y), color);	// `````` Segment 6
			PutPixel((Guint)(b+x), (Guint)(y-a), color);	// `````` Segment 2
			PutPixel((Guint)(a+x), (Guint)(y-b), color);	// `````` Segment 1
			PutPixel((Guint)(x-a), (Guint)(y-b), color);	// `````` Segment 8
			PutPixel((Guint)(x-b), (Guint)(y-a), color);	// `````` Segment 7
		}

		if(P < 0)    P += 3 +2 *a++;
    else         P += 5 +2 *(a++ - b--);
	} while(a <= b);
}	//Circle_Fill

// Description	: Draw a ellipse on the screen. 
//							: base on http://homepage.smc.edu/kennedy_john/belipse.pdf
// Input       	: x,y  - the center of the ellipse
//             	: Xrad - defines the X-redius of the ellipse,
//             	: Yrad - defines the Y-redius of the ellipse,
//             	: fill - fill yes or no
//********************************************************************
void Ellipse_Fill(Guint x, Guint y, Guint Xradius, Guint Yradius, unsigned char fill, Guint color)
{
	signed int 		a, b, c, Pb, Pc;

  signed int		XChange, YChange;
  signed int		StoppingX, StoppingY;
  signed int		EllipseError;

  b  = Xradius;
  c  = Yradius;
  Pb = 2 *Xradius *Xradius;
  Pc = 2 *Yradius *Yradius;
	
	//---------------------------		first set of points
	a 								= 0;
  XChange 					= Yradius *Yradius *(1 -2 *Xradius);
  YChange 					= Xradius *Xradius;
  EllipseError			= 0;
  StoppingX 				= Pc *Xradius;
  StoppingY					= 0;
 
	do
	{
   	if(fill)
    {
			RectangleFill((Guint)(x-b), (Guint)(y+a), (Guint)(x+b), (Guint)(y+a), color);	// 6 --> 3
			RectangleFill((Guint)(x-b), (Guint)(y-a), (Guint)(x+b), (Guint)(y-a), color);	// 7 --> 2
    }
    else
    {
																										//     			8		1
																										//     	7						2
																										//     	6						3
																										//     			5		4
																										//
			PutPixel((Guint)(b+x), (Guint)(a+y), color);	// `````` Segment 3
			PutPixel((Guint)(x-b), (Guint)(a+y), color);	// `````` Segment 6
			PutPixel((Guint)(b+x), (Guint)(y-a), color);	// `````` Segment 2
			PutPixel((Guint)(x-b), (Guint)(y-a), color);	// `````` Segment 7
		}
		
		//--------------
		a								+= 1;
		StoppingY 			+= Pb;
		EllipseError 		+= YChange;
		YChange 				+= Pb;
    
		if((2 *EllipseError +XChange) > 0)
		{
			b							-= 1;
      StoppingX 		-= Pc;
      EllipseError	+= XChange;
      XChange 			+= Pc;
    }
	} while(StoppingX >= StoppingY);


	//---------------------------		second set of points
  a 								= 0;
  YChange 					= Xradius *Xradius *(1 -2 *Yradius);
  XChange 					= Yradius *Yradius;
  EllipseError 			= 0;
  StoppingY 				= Pb *Yradius;
  StoppingX 				= 0;

 	do
  {
   	if(fill)
    {
			RectangleFill((Guint)(x-a), (Guint)(y+c), (Guint)(x+a), (Guint)(y+c), color);	// 5 --> 4
			RectangleFill((Guint)(x-a), (Guint)(y-c), (Guint)(x+a), (Guint)(y-c), color);	// 8 --> 1
    }
    else
    {
																										//     			8		1
																										//     	7						2
																										//     	6						3
																										//     			5		4
																										//
			PutPixel((Guint)(a+x), (Guint)(c+y), color);	// `````` Segment 4
			PutPixel((Guint)(x-a), (Guint)(c+y), color);	// `````` Segment 5
			PutPixel((Guint)(a+x), (Guint)(y-c), color);	// `````` Segment 1
			PutPixel((Guint)(x-a), (Guint)(y-c), color);	// `````` Segment 8
		}

		//--------------
		a								+= 1;
		StoppingX 			+= Pc;
		EllipseError 		+= XChange;
		XChange 				+= Pc;
		
		if((2 *EllipseError +YChange) > 0)
		{
			c							-= 1;
			StoppingY 		-= Pb;
			EllipseError 	+= YChange;
			YChange 			+= Pb;
		}
	} while(StoppingY >= StoppingX);
}	//Ellipse_Fill
