//-----------------------------------------------------------------------------
// Copyright:     	RAD Electronic Co. LTD,
// Author:         	Sh. Nourbakhsh Rad
// Remarks:        	Based on Alex Hiam(ahiam@marlboro.edu) menu system
// known Problems: 	None
// Version:        	1.5.0
// Description:    	Menu based user interface framework
//-----------------------------------------------------------------------------

#include "Menu.h"


//******************* Constants
MenuItem 												MenuItems[MAX_ITEMS];

unsigned char 									UpVal;
unsigned char 									DownVal;
unsigned char 									SelectVal;

unsigned char										UpPressed;
unsigned char										DownPressed;
unsigned char										SelectPressed;
unsigned char										selFlag;

unsigned char 									UpHoldCounter;
unsigned char 									DownHoldCounter;

unsigned char 									CurrentLine;
unsigned char										ayOLD;

unsigned char 									NoItems;
unsigned char 									ItemIndex;

signed char 										DrawIndex;

int 														ActiveTimeout;
int 														InactiveCount;

unsigned char										HasDrawFunct;
unsigned char										HasTimeoutFunct;

char 														Caption[LABEL_LEN];
char 														lMenu[5];
char 														rMenu[5];

//----------------------------
void (*DrawFunction)(void);
void (*TimeoutFunction)(void);



//*************************************************
//******************* Functions *******************
//*************************************************
void Menu_Scroll(signed char dir)
{
  // Only scrolls screen if possible, doesn't change ItemIndex
  DrawIndex = DrawIndex +dir;
  if(DrawIndex < 0)					DrawIndex = 0;
  
  if(NoItems >= NO_LINES)
  {
    if(DrawIndex > (NoItems -NO_LINES +1))				DrawIndex = NoItems -NO_LINES +1;

		Menu_Draw();
  }
  else
  {
  	DrawIndex = 0;
  }
}	//Menu_Scroll

void Menu_DrawHLs(void)
{
  unsigned char							ay;
  char											chr;

  if(NoItems)
  {
		// Draw arrow to indicate current item
		#if (HighlightMode ==0)
			ay = CurrentLine +1;

			if(ayOLD !=ay)
			{
				if(ayOLD)
				{
					GLCDcolors(WHITE, WHITE);
					GLCDchar(LEFT_MARGIN -7, ayOLD, LeftArrow);
				}
				ayOLD = ay;
				
				GLCDcolors(BLACK, WHITE);
				GLCDchar(LEFT_MARGIN -7, ay, LeftArrow);
			}
				
		#elif (HighlightMode ==1)
			ay = GLCDFontH +(GLCDFontH *CurrentLine) -1;
			
			if(ayOLD !=ay)
			{
				if(ayOLD)				GLCDinvrect(LEFT_MARGIN -2, ayOLD, HLwidth, GLCDFontH -1);
				ayOLD = ay;
				
				GLCDinvrect(LEFT_MARGIN -2, ay, HLwidth, GLCDFontH -1);
			}
			
		#else
			ay = GLCDFontH +(GLCDFontH /2) +(GLCDFontH *CurrentLine) -1;
			
			if(ayOLD !=ay)
			{
				if(ayOLD)
				{
					GLCDline(0,
									 ayOLD,
									 LEFT_MARGIN -5,
									 ayOLD,
									 WHITE);
				  						 
					GLCDline(LEFT_MARGIN -5,
									 ayOLD -3,
									 LEFT_MARGIN -2,
									 ayOLD,
									 WHITE);
				  						 
					GLCDline(LEFT_MARGIN -5,
									 ayOLD +3,
									 LEFT_MARGIN -2,
									 ayOLD,
									 WHITE);
											 
					GLCDline(LEFT_MARGIN -5,
									 ayOLD +3,
									 LEFT_MARGIN -5,
									 ayOLD -3,
									 WHITE);
				}
				ayOLD = ay;

				GLCDline(0,
								 ay,
								 LEFT_MARGIN -5,
								 ay,
								 BLACK);
			  						 
				GLCDline(LEFT_MARGIN -5,
								 ay -3,
								 LEFT_MARGIN -2,
								 ay,
								 BLACK);
			  						 
				GLCDline(LEFT_MARGIN -5,
								 ay +3,
								 LEFT_MARGIN -2,
								 ay,
								 BLACK);
										 
				GLCDline(LEFT_MARGIN -5,
								 ay +3,
								 LEFT_MARGIN -5,
								 ay -3,
								 BLACK);
			}

		#endif
	
		// Draw up arrow if there are items above view
		if(DrawIndex > 0)
		{
			#if (HighlightMode ==0)
				GLCDchar(ArrowX, 1, UpArrow);
			#else
				GLCDline(ArrowX1, ArrowY2, ArrowX1 +2, ArrowY2 +3, BLACK);
				GLCDline(ArrowX2, ArrowY2, ArrowX2 -2, ArrowY2 +3, BLACK);
			#endif
	  }
	
		// Draw down arrow if there are items below view
		if((NoItems -DrawIndex) >= NO_LINES)
		{
			#if (HighlightMode ==0)
				GLCDchar(ArrowX, NO_LINES -1, DownArrow);

			#else
				GLCDline(ArrowX1, ArrowY1, ArrowX1 +2, ArrowY1 -3, BLACK);
				GLCDline(ArrowX2, ArrowY1, ArrowX2 -2, ArrowY1 -3, BLACK);
			#endif
		}
	}//if NoItems

	if(HasDrawFunct)						DrawFunction();
}	//Menu_DrawHLs

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void Menu_Init(unsigned char UpKeyCode, unsigned char DownKeyCode, unsigned char SelectKeyCode)
{
	Menu_SetKeys(UpKeyCode, DownKeyCode, SelectKeyCode);
  Menu_Clear();
}	//Menu_Init

void Menu_Clear(void)
{
  // Reset all flags/counters
  UpPressed				= False;
  DownPressed 		= False;
  SelectPressed 	= False;
 	selFlag 				= False;


  Caption[0]			= '\0';
  lMenu[0]				= '\0';
  rMenu[0]				= '\0';
  
  NoItems 				= 0;
  CurrentLine 		= 0;
  ayOLD						= 0;
  DrawIndex 			= 0;
  ItemIndex 			= 0;
  
  HasDrawFunct 		= False;
  HasTimeoutFunct = False;
  
  InactiveCount 	= 0;
  ActiveTimeout 	= 0;

	GLCDclear();
}	//Menu_Clear

void Menu_SetKeys(unsigned char UpKeyCode, unsigned char DownKeyCode, unsigned char SelectKeyCode)
{
	// Define input keys code
	UpVal 					= UpKeyCode;
  DownVal					= DownKeyCode;
  SelectVal				= SelectKeyCode;
}	//Menu_SetKeys

//---------------------------------------
void Menu_Update(unsigned char inKey)
{
	unsigned char				nLine = 1;
	
  //----------------	U P
	if(inKey == UpVal)
	{
		if(!UpPressed)												// Only move once until button release or MAX_HOLD_COUNT reached
		{
			if((strcmp(MenuItems[ItemIndex -1].label, NULL) ==0) &&
				 (strlen(MenuItems[ItemIndex -1].label) ==0))					 				nLine += 1;

			if(CurrentLine > (nLine -1))
			{
				CurrentLine -= nLine;
			}
      else
      {
				if(CurrentLine == (nLine -1))				 	CurrentLine -= (nLine -1);
				Menu_Scroll(-nLine);							// We're at the top line, scroll up if possible
			}
       
      if(ItemIndex > 0)	        	ItemIndex -= nLine;
      
      UpPressed = True; 
  	
  		Menu_DrawHLs();
    }
    else
    {
    	UpHoldCounter++;
    }
    
    if(UpHoldCounter >= MAX_HOLD_COUNT)
    {
      UpPressed = False;
      UpHoldCounter = HOLD_SCROLL_SPEED;
    }

  	selFlag = False;
  }
  else
  {
    UpPressed = False;
    UpHoldCounter = 0;
  }
  
  //----------------	D O W N
	if(inKey == DownVal)
  {
    if(!DownPressed)											// Wait until button release or max hold time reached
    {
			if((strcmp(MenuItems[ItemIndex +1].label, NULL) ==0) &&
				 (strlen(MenuItems[ItemIndex +1].label) ==0))					 				nLine += 1;

      if((CurrentLine < (NO_LINES -(nLine +1))) & (CurrentLine < (NoItems -nLine)))
      {
      	CurrentLine += nLine;
      }
      else
      {
      	if(CurrentLine == (NO_LINES -(nLine +1)))      		CurrentLine += (nLine -1);
      	Menu_Scroll(nLine); 							// We're at the bottom line, scroll down if possible
      }

      if(ItemIndex < (NoItems -1))	        ItemIndex += nLine;
      
      DownPressed = True; 

  		Menu_DrawHLs();
    }
    else
    {
    	DownHoldCounter++;
    }
    
    if(DownHoldCounter >= MAX_HOLD_COUNT)
    {
      DownPressed = False;
      DownHoldCounter = HOLD_SCROLL_SPEED; 
    }
  	
  	selFlag = False;
  }
  else
  {
    DownPressed = False;
    DownHoldCounter = 0;
  }

  //----------------	S E L E C T
	if(inKey == SelectVal)
  {
    if(!SelectPressed)										// Only select once until button release
    {
      if(MenuItems[ItemIndex].pass_value)
      {
        MenuItems[ItemIndex].value_function(MenuItems[ItemIndex].value);
      }
      else if (MenuItems[ItemIndex].funct)
      {
      	MenuItems[ItemIndex].function();
      }
      
      SelectPressed = True;
    }
 		
 		if(selFlag)  				Menu_DrawHLs();
 		else								Menu_Draw();
  	
   	selFlag = True;
  }
  else
  {
  	SelectPressed = False;
  }
  
  //---------------- T I M E O U T 
  if(HasTimeoutFunct)
  {
		// Inactive this time through loop
		if(!(UpPressed | DownPressed | SelectPressed))				InactiveCount++;
		else 																									InactiveCount = 0;  		// Reset counter if any button was pressed
    
		if(InactiveCount >= ActiveTimeout)
		{
			// Timeout reached, call timeout funtion
			TimeoutFunction();
			InactiveCount = 0;

  		Menu_Draw();
		}
	}
}	//Menu_Update

void Menu_Draw(void)
{
  unsigned char 					i;
  
  GLCDclear();
  
  GLCDstring(0, 0, Caption);
  Menu_Footer(lMenu, rMenu);

  if(NoItems)
  {
	 	GLCDstring(LEFT_MARGIN, i +1, MenuItems[DrawIndex +i].label);
	
	  i = 0;
	  while((i < (NO_LINES -1)) & (i < NoItems))
	  {
	    GLCDstring(LEFT_MARGIN, i +1, MenuItems[DrawIndex +i].label);
	    i++;
	  }
	}//if NoItems
	
	ayOLD = 0;
	Menu_DrawHLs();
}	//Menu_Draw

//---------------------------------------
void Menu_SetCaption(char *label)
{
	strcpy(Caption, label);									// First line of display (blank if not set)
}	//Menu_SetCaption

void Menu_SetFooter(char *lText, char *rText)
{
	if(lText)			strcpy(lMenu, lText);
	else					*lMenu = NULL;

	if(rText)			strcpy(rMenu, rText);
	else					*rMenu = NULL;
}	//Menu_SetFooter

void Menu_Footer(char *lText, char *rText)
{
	if(lText)
	{
		if(*lText !=NULL)
		{			
			GLCDstring(10, 6, lText);
			GLCDinvrect(2, 52, 60, 10);
		}
	}
	
	//-----------
	if(rText)
	{
		if(*rText != NULL)
		{			
			GLCDstring(96, 6, rText);
			GLCDinvrect(64, 52, 60, 10);
		}
	}
}	//Menu_Footer

//---------------------------------------
void Menu_AddItem1(char *label)
{
	MenuItem 				i;
	
	if(NoItems >= MAX_ITEMS) 						return;
	if(strlen(label) > LABEL_LEN)				return;
	
	strcpy(i.label, label);
	
	i.pass_value 			= False;
	i.funct 					= False; 							// Calls no function
	
	MenuItems[NoItems++] = i;
}	//Menu_AddItem1

void Menu_AddItem2(char *label, void (*function)(void))
{
	MenuItem 				i;
	
	if(NoItems >= MAX_ITEMS) 						return;
	if(strlen(label) > LABEL_LEN)				return;
	
	strcpy(i.label, label);
	
	i.pass_value 			= False;							// No value to pass to function
	i.funct 					= True; 							// Has function and should pass it a value
	i.function 				= function;						// Function to call
	
	MenuItems[NoItems++] = i;
}	//Menu_AddItem2

void Menu_AddItem3(char *label, int value, void (*function)(int))
{
	MenuItem 				i;
	
	if(NoItems >= MAX_ITEMS) 						return;
	if(strlen(label) > LABEL_LEN)				return;
	
	strcpy(i.label, label);
	
	i.pass_value 			= True;
	i.funct 					= True; 							// Has function and should pass it a value
	i.value 					= value;							// Value to pass to function on select
	i.value_function 	= function;						// Function to call
	
	MenuItems[NoItems++] = i;
}	//Menu_AddItem3

//---------------------------------------
void Menu_AddDrawFunction(void (*function)(void))
{
  HasDrawFunct = True;
  DrawFunction = function;
}	//Menu_AddDrawFunction

void Menu_AddTimeoutFunction(int timeout, void (*function)(void))
{
	HasTimeoutFunct = True;
	ActiveTimeout = timeout;								// Number of loops without button press
																					// before calling timeout function
	TimeoutFunction = function;
}	//Menu_AddTimeoutFunction

void Menu_RemoveDrawFunction(void)
{
  HasDrawFunct = False;
  DrawFunction = 0;
}	//Menu_RemoveDrawFunction

void Menu_RemoveTimeoutFunction(void)
{
	HasTimeoutFunct = False;
	ActiveTimeout = 0;
	TimeoutFunction = 0;
}	//Menu_AddTimeoutFunction
