//-----------------------------------------------------------------------------
// Copyright:     	RAD Electronic Co. LTD,
// Author:         	Sh. Nourbakhsh Rad
// Remarks:        	Based on Alex Hiam(ahiam@marlboro.edu) menu system
// known Problems: 	None
// Version:        	1.5.0
// Description:    	Menu based user interface framework
//-----------------------------------------------------------------------------

#ifndef _MENU_H_
	#define _MENU_H_

  #include "../app_config.h"
  
  #include "../GLCD/ks0108.h"
  #include "../Graphic/Graphic.h"
  #include "../FONT/Font.h"
  

	//----------- Definitions and Constants
	#define LEFT_MARGIN       							10	// Should be >4
	#define MAX_ITEMS         							12
	#define MAX_HOLD_COUNT    							3		// Loops of holding up or down button
                           										// before continuing to scroll 
	#define HOLD_SCROLL_SPEED 							3
	
	//-----------
	#define NO_LINES 												7
	#define LABEL_LEN 											16

	//-----------
	#define HighlightMode										0		// 0: Text arrow mode
																							// 1: Line invers mode
																							// 2: Graphic arrow mode

	#if (HighlightMode ==0)
		#define ArrowX												120
		
		#define LeftArrow											0xBB //0x3E , 0x7E
		#define UpArrow												0x86 //
		#define DownArrow											0x87 //
	
	#elif (HighlightMode >0)
		#if (HighlightMode ==1)
			#define HLwidth											110
		#endif
	
		#define ArrowX1												120
		#define ArrowX2												124
		
		#define ArrowY1												12
		#define ArrowY2												56

	#endif


	//----------- GLCD functions ...
	#define GLCDclear()																{KS108_CLSx();}
	#define GLCDinvrect(x, y, w, h)										{KS108_InvertRect(x, y, w, h);}
	
	#define GLCDline(x1, y1, x2, y2, clr)							{Line(x1, y1, x2, y2, clr);}
	
	#define GLCDcolors(fk, bk)												{SetFgColor(fk); SetBkColor(bk);}
	#define GLCDstring(x, l, str)											{SetLineX(l,x); Puts(str);}
	#define GLCDchar(x, l, chr)												{SetLineX(l,x); PutChar(chr);}

	#define GLCDFontW					 												((FontWidth  +FontSpaceX) *FontXScale)
	#define GLCDFontH					 												((FontHeight +FontSpaceY) *FontYScale)

	//----------- Struct used for all types of menu items
	typedef struct {
			char 							label[LABEL_LEN +1];
			int 							value;
			int								update_counter;
			unsigned char			pass_value;
			unsigned char			funct;
			void 							(*function)(void);
			char*							(*data_function)(void);
			void 							(*value_function)(int);
		} MenuItem;

	
	//******************* Function Prototypes
	void Menu_Init(unsigned char UpKeyCode, unsigned char DownKeyCode, unsigned char SelectKeyCode);
	
	void Menu_SetKeys(unsigned char UpKeyCode, unsigned char DownKeyCode, unsigned char SelectKeyCode);
	void Menu_Clear(void);
	
	void Menu_Update(unsigned char inKey);
	void Menu_Draw(void);
	
	void Menu_SetCaption(char *label);
	void Menu_SetFooter(char *lText, char *rText);
	void Menu_Footer(char *lText, char *rText);
	
	void Menu_AddItem1(char *label);
	void Menu_AddItem2(char *label, void (*function)(void));
	void Menu_AddItem3(char *label, int value, void (*function)(int));
	
	void Menu_AddDrawFunction(void (*function)(void));
	void Menu_AddTimeoutFunction(int timeout, void (*function)(void));
	
	void Menu_RemoveTimeoutFunction(void);
	void Menu_RemoveDrawFunction(void);

#endif	//_MENU_H_
