//-----------------------------------------------------------------------------
// Copyright:      RAD Electronic Co. LTD,
// Author:         Sh. Nourbakhsh Rad
// Remarks:        
// known Problems: none
// Version:        1.2.0
// Description:    AVR menu-system test ....
//								 
//								 
//-----------------------------------------------------------------------------

//	Include Headers
//*****************************************************************************
#include "app_config.h"
#include "myHW.h"

//--------------------------------------------------
#include "GLCD\ks0108.h"
#include "Graphic\Graphic.h"

//--------------------------------------------------
#include "FONT\Font.h"

//#include "FONT\font\f9x14.h"
//#include "FONT\font\AF12x16.h"
//#include "FONT\font\AF9x11.h"
#include "FONT\font\f5x7.h"

//--------------------------------------------------
#include "4x4Key\4Key.h"

#include "Menu\Menu.h"



//	Constants and Varables
//*****************************************************************************
//
	
//Temp Const.
char 														Ctemp[24];
int  														Itemp = 0;


//Backlight PWM values:
unsigned char brightness_levels[7] = { 0, 20, 50, 90, 130, 190, 250 };
unsigned char brightness_index; 															// Index of brightness_levels


//	Functions Prototype
//*****************************************************************************
void Initial(void);
void Splash(void);

void testMN(unsigned char rFlag);

void show_main(void);
void show_analog(void);
void show_brightness(void);
void scroll_test(void);
void draw_analog(void);
void set_brightness(int dir);
void draw_brightness(void);
void light_off(void);
void loop(void);



//	<<< main function >>>
//*****************************************************************************
void main(void)
{
	Initial();
	
	testMN(0);
	
	while(1);
}	//main


//---------------------------------------------------------//
//---------------------------------------------------------//
void Initial(void)
{
	cli();																	//Interrupts disable
	HW_init();
	_delay_ms(20);

	//----------------------
	KS108_Init(NON_INVERTED);
	KS108_Backlight(20);										//PWM mode!
	LcdNonTransparence(ON);

	LcdFont(f5x7);													//This font not have height space!
	LcdFontSpaceY(2);

	//----------------------
	KEY4_init();
	
	//----------------------
	_delay_ms(200);
	LED_signal(BOOT_sign);

	KS108_CLSx();
	
	//----------------------
	Splash();
	LED_signal(OK_sign);

	//------------
	//sei();																//Interrupts enabel	
}	//Initial

void Splash(void)
{
	static char *SPchar[3] = {
			"GLCD Menu system!",
			"by :",
			"   Sh. Nourbakhsh",
			};

	//-------------
	LcdFontYScale(2);
	SetCursor(10, 8);				Puts(SPchar[0]);

	LcdFontYScale(1);
	DrawStringAt(4, 1, SPchar[1], BLACK, WHITE);
	DrawStringAt(5, 1, SPchar[2], BLACK, WHITE);

	//-------------
	_delay_ms(3000);
	KS108_CLSx();
}	//Splash


//---------------------------------------------------------//
//---------------------------------------------------------//
void testMN(unsigned char rFlag)
{
	unsigned char				oldKey;
	
	if(rFlag)
	{
		KS108_CLSx();
	}
	else
	{
		Menu_Init(KeyUP, KeyDN, KeyRM);
  
  	brightness_index = 1; 																		// Initial brightness
  	set_brightness(0);
	}

 	Menu_Footer("Menu", NULL);

	while(1)
	{
	 	sprintf(Ctemp, "Press KeyLM: 0x%02X", KEY4val);
	 	DrawStringAt(1, 1, Ctemp, BLACK, WHITE);
		
	 	sprintf(Ctemp, "Counter: %05u !", Itemp++);
	 	DrawStringAt(3, 1, Ctemp, BLACK, WHITE);

		//----------------------
		if(IS_Key_Press)
		{
	 		ISR_KEY4_INT();
			
			if(KEY4val ==KeyLM)
			{
				show_main();
				Menu_Draw(); 																					// Draw main menu

				do
				{
					ISR_KEY4_INT();
					Menu_Update(KEY4val);

					_delay_ms(50);
				}while(KEY4val !=KeyLM);

				KEY4val = 0x00;
				
				KS108_CLSx();
 				Menu_Footer("Menu", NULL);
			}//if KEY4val = KeyLM
		}//if IS_Key_Press

		_delay_ms(100);
	}//while 1
}	//testMN

//-----------	Draw main menu screen
void show_main(void)
{
  Menu_Clear(); 																							// Clear menu and display
  
	Menu_RemoveTimeoutFunction();																// Remove last timeout function
  Menu_AddTimeoutFunction(200, light_off); 										// If Menu_Update called 200 times, light_off() will be called

  Menu_SetCaption("Example Menu:");														// Title always shown on first line:
  
  //----	Add items with function to be called when selected:
  Menu_AddItem2("Read Analog", show_analog);
  Menu_AddItem2("Set Brightness", show_brightness);
  Menu_AddItem2("Scroll Test", scroll_test);

	Menu_SetFooter("Back", "Slct");
}	//show_main

//-----------	Draw analog display menu
void show_analog(void)
{
 	Menu_Clear(); 
  
  Menu_AddDrawFunction(draw_analog);
  Menu_SetCaption("Analog Value:");

 	Menu_SetFooter("Back", NULL);
}	//show_analog

//-----------	Draw brightness menu
void show_brightness(void)
{
  Menu_Clear();
  
  Menu_AddTimeoutFunction(150, show_main);										// Back to main menu after 150 a inactive loops
  
  Menu_AddDrawFunction(draw_brightness);
  Menu_SetCaption("Brightness:");
  
  Menu_AddItem3("Up", 1, set_brightness);											//pass (1 and -1) to set_brightness
  Menu_AddItem3("Down", -1, set_brightness);
  Menu_AddItem1(NULL);
  Menu_AddItem2("Back", show_main);
  
 	Menu_SetFooter(NULL, NULL);
}	//show_brightness

//-----------	Fill a menu to demonstrate scrolling
void scroll_test(void)
{
  Menu_Clear();
  
  Menu_AddTimeoutFunction(250, show_main);
  Menu_SetCaption("Scroll Test:");
  
  Menu_AddItem2("Back", show_main);
  Menu_AddItem1(NULL);
	Menu_AddItem1("Menu Item 01");
	Menu_AddItem1("Menu Item 02");
	Menu_AddItem1("Menu Item 03");
	Menu_AddItem1("Menu Item 04");
	Menu_AddItem1("Menu Item 05");
	Menu_AddItem1("Menu Item 06");
	Menu_AddItem1("Menu Item 07");
	Menu_AddItem1("Menu Item 08");
  Menu_AddItem1(NULL);
  Menu_AddItem2("Back", show_main);
  
 	Menu_SetFooter(NULL, NULL);
}	//scroll_test

//---------------------------------------------------------
//-----------	Convert and display ADC values from A0 and A1:
void draw_analog(void)
{
  unsigned int 					value, i;
  
  do
  {
	  value = 136 +i;																						//for example: ADC-Read(A0);
	  sprintf(Ctemp, "A0: %03u", value);
	 	DrawStringAt(2, 3, Ctemp, BLACK, WHITE);
	
	  value = 121 +i;																						//for example: ADC-Read(A1);
	  sprintf(Ctemp, "A1: %03u", value);
	 	DrawStringAt(3, 3, Ctemp, BLACK, WHITE);
	
		ISR_KEY4_INT();
		_delay_ms(100);
		i++;
	}while(KEY4val !=KeyLM);
	
	KEY4val = 0x00;

	show_main();
	Menu_Draw();
}	//draw_analog

//-----------	Draw lower section of brightness screen:
void draw_brightness(void)
{
  unsigned char			x1  = LEFT_MARGIN +17;
  unsigned char			x21 = x1 +12 *6;
  unsigned char			x22 = x1 +12 *brightness_index;
  unsigned char			y1  = 44 +8;
  unsigned char			y2  = 52 +8;
  
  // Just display brightness_index as brightness level:
  sprintf(Ctemp, "%02u", brightness_index);
 	DrawStringAt(6, LEFT_MARGIN +9, Ctemp, BLACK, WHITE);
  
  // Visual level display:
 	RectangleFill(x1, y1, x21, y2, WHITE);											// Clear last box
 	
 	Rectangle(x1, y1, x21, y2, BLACK);
 	RectangleFill(x1, y1, x22, y2, BLACK);
}	//draw_brightness

//-----------	Lower brightness if dir<0, raise if dir>0
void set_brightness(int dir)
{
	if(dir < 0)				{ if(brightness_index > 0)				brightness_index--;	}
	if(dir > 0)				{ if(brightness_index < 6)				brightness_index++; }

	KS108_Backlight(brightness_levels[brightness_index]);
}	//set_brightness

//-----------	
void light_off(void)
{
  unsigned char 			brightness = brightness_index; 					// Save current level
  
  brightness_index = 0;
  set_brightness(0);
  
  // Hold until button press then reset brightnes
  while(!IS_Key_Press);
  
  brightness_index = brightness;
  set_brightness(0);
}	//light_off
